<?php

// DB-phpof.php - abstract database functions and make them easier to use
// Attempts to retail syntactic compatibility with PEAR DB class.
//
// $Id: DB-phpof.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2001-2006 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require_once('XError.class.php');

// Event types, for audit logging
define('PHPOF_EVENT_DELETE','delete');
define('PHPOF_EVENT_INSERT','insert');
define('PHPOF_EVENT_UPDATE','update');
define('PHPOF_EVENT_NOTE','note');

// Binary data handling
define('PHPOF_BINARYDATA_PRESENT','!placeholder!');
define('PHPOF_BINARYDATA_DELETE','!delete!');
define('PHPOF_BINARYDATA_UNLIMITED',0);
define('PHPOF_BINARYDATA_ANY_TYPE','any');

// Returned by functions when not supported by a database
define('PHPOF_NOTSUPPORTED',-1);
// Returned by functions when disabled by a config option
define('PHPOF_DISABLED',-2);

// Used in DB_result::getArray
define('PHPOF_ARRAY_LINEAR',-1);

define('PHPOF_USE_DEFAULT_ORDER','');
define('PHPOF_USE_DEFAULT_DN',-1);
define('PHPOF_USE_KEY',-1);

if ($PHPOF_TABLE_CACHING && !$PHPOF_TABLE_CACHING_TYPE) {
	$PHPOF_TABLE_CACHING_TYPE = 'gdbm';
}

/**
 * This class is used as a top-level 'martialling' class for handling databases.
 * Basically, you call DB::connect without initialising a DB object, and this
 * returns an initialised object of type DB_xxx, where xxx is the type of 
 * database you're using.
 *
 */
class DB {
	var $_dbh; // Database handle

	/**
	 * Constructor; not really used unless for some reason you want to create
	 * a new DB object with an existing DB handle
	 */ 
	function DB ($dbh) {
		$this->_dbh = $dbh;
	}

	/**
	 * Connect to the database server and select the database we wish to use
	 * 
	 * @param string $url  A database URI of the form 'dbtype://username:password@host/database_name'.
	 * @return mixed  An initialised DB object of type DB_xxx
	 */ 
	function connect ($url) {
		global $PHPOF_DB_BACKEND;
		ereg("(.*):\/\/(.*):(.*)@(.*)\/(.*)",$url, $matches);
		$dbtype = $matches[1];
		$username = $matches[2];
		$password = $matches[3];
		$dbhost = $matches[4];
		$dbname = $matches[5];
		
		if (empty($PHPOF_DB_BACKEND) || $PHPOF_DB_BACKEND == 'phpof') {
			include_once("dbmodules/DB_$dbtype.class.php");
			$dbclass = 'DB_'.$matches[1];
			$d = new $dbclass ($dbhost, $username, $password, $dbname);
		} else if ($PHPOF_DB_BACKEND == 'adodb') {
			include_once('dbmodules/DB_adodb.class.php');
			$dbclass = 'DB_adodb';
			$d = new DB_adodb ($dbhost, $username, $password, $dbname, $dbtype);
		} else if ($PHPOF_DB_BACKEND == 'sqlrelay') {
			include_once('dbmodules/DB_sqlrelay.class.php');
			$dbclass = 'DB_sqlrelay';
			$d = new DB_sqlrelay ($dbhost, $username, $password, $dbname, $dbtype);
		} else {
			trigger_error("Unknown database backend '$PHPOF_DB_BACKEND'",E_USER_ERROR);
			die();
		}
		
		return $d;
	}
	
	/**
	 * Create a table object of a specified name, respecting the PHPOF_TABLE_CLASS_MAPPINGS option
	 *
	 * @param string $TableName  Name (in database) of table to create
	 * @return DBTable  Instantiated table object
	 */
	function createTableObject($TableName)
	{
		global $PHPOF_TABLE_CLASS_MAPPINGS;
		
		if (!empty($PHPOF_TABLE_CLASS_MAPPINGS[$TableName])) {
			$ObjectName = $PHPOF_TABLE_CLASS_MAPPINGS[$TableName];
			return new $ObjectName;
		} else {
			return new $TableName;
		}
	}
	
}