<?php

// MIMETypeHelper.class.php - PHPOF MIME helper class
//
// $Id: MIMETypeHelper.class.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2003-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// In case we are not in global scope
global $MIME_TYPE_MAPPINGS, $MIME_TYPE_NAMES;

// Put the PRIMARY mapping first (for reverse lookups)
$MIME_TYPE_MAPPINGS = array(
		'bmp'=>'image/bmp',
		'css'=>'text/css',
		'csv'=>'text/csv',
		'doc'=>'application/msword',
		'docm'=>'application/vnd.ms-word.document.macroEnabled.12',
		'docx'=>'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
		'dotm'=>'application/vnd.ms-word.template.macroEnabled.12',
		'dotx'=>'application/vnd.openxmlformats-officedocument.wordprocessingml.template',
		'exe'=>'application/octet-stream',
		'flac'=>'audio/flac',
		'flv'=>'video/x-flv',
		'gif'=>'image/gif',
		'htc'=>'text/x-component',
		'htm'=>'text/html',
		'html'=>'text/html',
		'inc'=>'application/x-httpd-php',
		'jpeg'=>'image/jpeg',
		'jpg'=>'image/jpeg',
		'jpe'=>'image/jpeg',
		'tpl'=>'text/html',
		'js'=>'application/x-javascript',
		'mov'=>'video/quicktime',
		'mp3'=>'audio/mpeg',
		'mpeg'=>'video/mpeg',
		'mpg'=>'video/mpeg',
		'mpe'=>'video/mpeg',
		'odt'=>'application/vnd.oasis.opendocument.text',
		'oga'=>'audio/ogg',
		'ogg'=>'audio/ogg',
		'ogv'=>'video/ogg',
		'ogx'=>'application/ogg',
		'pdf'=>'application/pdf',
		'php'=>'application/x-httpd-php',
		'png'=>'image/png',
		'potm'=>'application/vnd.ms-powerpoint.template.macroEnabled.12',
		'potx'=>'application/vnd.openxmlformats-officedocument.presentationml.template',
		'ppam'=>'application/vnd.ms-powerpoint.addin.macroEnabled.12',
		'ppsm'=>'application/vnd.ms-powerpoint.slideshow.macroEnabled.12',
		'ppsx'=>'application/vnd.openxmlformats-officedocument.presentationml.slideshow',
		'pptm'=>'application/vnd.ms-powerpoint.presentation.macroEnabled.12',
		'pptx'=>'application/vnd.openxmlformats-officedocument.presentationml.presentation',
		'ppt'=>'application/vnd.ms-powerpoint',
		'qt'=>'video/quicktime',
		'rar'=>'application/x-rar-compressed',
		'rpm'=>'application/x-rpm',
		'rtf'=>'text/rtf',
		'sldm'=>'application/vnd.ms-powerpoint.slide.macroEnabled.12',
		'sldx'=>'application/vnd.openxmlformats-officedocument.presentationml.slide',
		'swf'=>'application/x-shockwave-flash',
		'sxw'=>'application/vnd.sun.xml.writer',
		'tiff'=>'image/tiff',
		'tif'=>'image/tiff',
		'txt'=>'text/plain',
		'vsd'=>'application/vnd.visio',
		'wav'=>'audio/wav',
		'xlam'=>'application/vnd.ms-excel.addin.macroEnabled.12',
		'xls'=>'application/vnd.ms-excel',
		'xlsb'=>'application/vnd.ms-excel.sheet.binary.macroEnabled.12',
		'xlsm'=>'application/vnd.ms-excel.sheet.macroEnabled.12',
		'xlsx'=>'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
		'xltm'=>'application/vnd.ms-excel.template.macroEnabled.12',
		'xltx'=>'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
		'xml'=>'text/xml',
		'zip'=>'application/zip',
);

$MIME_TYPE_NAMES = array(
		'application/msword'=>'Microsoft Word document',
		'application/octet-stream'=>'Binary file',
		'application/ogg' => 'Ogg multiplex profile',
		'application/pdf' => 'PDF document',
		'application/vnd.ms-excel.addin.macroEnabled.12' => 'Microsoft Office Excel 2007 add-in',
		'application/vnd.ms-excel' => 'Microsoft Excel file',
		'application/vnd.ms-excel.sheet.binary.macroEnabled.12' => 'Microsoft Office Excel binary workbook',
		'application/vnd.ms-excel.sheet.macroEnabled.12' => 'Microsoft Office Excel macro-enabled workbook',
		'application/vnd.ms-excel.template.macroEnabled.12' => 'Microsoft Office Excel macro-enabled workbook template',
		'application/vnd.ms-powerpoint.addin.macroEnabled.12' => 'Microsoft Office Powerpoint add-in',
		'application/vnd.ms-powerpoint' => 'Microsoft Powerpoint document',
		'application/vnd.ms-powerpoint.presentation.macroEnabled.12' => 'Microsoft Office Powerpoint macro-enabled presentation',
		'application/vnd.ms-powerpoint.slide.macroEnabled.12' => 'Microsoft Office PowerPoint 2007 macro-enabled slide',
		'application/vnd.ms-powerpoint.slideshow.macroEnabled.12' => 'Microsoft Office Powerpoint macro-enabled slideshow',
		'application/vnd.ms-powerpoint.template.macroEnabled.12' => 'Microsoft Office Powerpoint macro-enabledtemplate',
		'application/vnd.ms-word.document.macroEnabled.12' => 'Microsoft Office Word macro-enabled document',
		'application/vnd.ms-word.template.macroEnabled.12' => 'Microsoft Office Word template',
		'application/vnd.oasis.opendocument.text'=>'OpenOffice.org Text Document',
		'application/vnd.openxmlformats-officedocument.presentationml.presentation' => 'Office OpenXML presentation',
		'application/vnd.openxmlformats-officedocument.presentationml.slide' => 'Office OpenXML slide',
		'application/vnd.openxmlformats-officedocument.presentationml.slideshow' => 'Office OpenXML slide show',
		'application/vnd.openxmlformats-officedocument.presentationml.template' => 'Office OpenXML presentation template',
		'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' => 'Office OpenXML spreadsheet',
		'application/vnd.openxmlformats-officedocument.spreadsheetml.template' => 'Office OpenXML spreadsheet template',
		'application/vnd.openxmlformats-officedocument.wordprocessingml.document' => 'Office OpenXML word processor document',
		'application/vnd.openxmlformats-officedocument.wordprocessingml.template' => 'Office OpenXML word processor template',
		'application/vnd.sun.xml.writer'=>'OpenOffice.org Text Document',
		'application/vnd.visio' => 'Microsoft Visio file',
		'application/x-httpd-php'=>'PHP source code',
		'application/x-javascript'=>'Javascript',
		'application/x-rar-compressed' => 'RAR archive',
		'application/x-rpm' => 'RPM package',
		'application/x-shockwave-flash' => 'Macromedia Flash file',
		'application/zip' => 'Zip archive',
		'audio/flac'=>'FLAC audio file',
		'audio/mpeg' => 'MPEG audio',
		'audio/ogg' => 'Ogg Vorbis audio',
		'audio/wav' => 'WAV audio',
		'image/bmp'=>'BMP image',
		'image/gif'=>'GIF image',
		'image/jpeg'=>'JPEG image',
		'image/png'=>'PNG image',
		'image/tiff' => 'TIFF image',
		'text/css'=>'Cascading Style Sheet',
		'text/csv'=>'Comma Separated Values file',
		'text/html'=>'HTML file',
		'text/plain' => 'Plain text file',
		'text/rtf' => 'RTF document',
		'text/x-component' => 'HTML component',
		'text/xml' => 'XML file',
		'video/mpeg' => 'MPEG video',
		'video/ogg' => 'Ogg Theora video',
		'video/quicktime' => 'QuickTime video',
		'video/x-flv'=>'Macromedia Flash video',
);

// Canonicalisations to perform on MIME types returned from external helpers
$MIME_TYPE_TRANSFORMATIONS = array(
	'application/x-zip' => 'application/zip',
	'image/x-png' => 'image/png',
);

class MIMETypeHelper {
	/**
	* @desc Get MIME type for a given local file
	* @param $File Local (filesystem-relative) filename
	* @param $UseFilename Filename to use to for the purposes of looking at a file extension, if relevant
	* @return string MIME type
	*/
	function getTypeFromFile($File, $UseFilename=null)
	{
		global $PHPOF_DEBUG_MODE, $MIME_TYPE_MAPPINGS, $MIME_TYPE_TRANSFORMATIONS;
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("MIMETypeHelper::getTypeFromFile() called with file='$File', UseFilename='$UseFilename'");
		
		if (empty($File)) trigger_error('MIMETypeHelper::getTypeFromFile() called with no filename',E_USER_ERROR);
		
		if (!empty($UseFilename)) {
			$PathParts = pathinfo($UseFilename);
			if (isset($PathParts['extension']) && !empty($PathParts['extension'])) {
				$Extension = strtolower($PathParts['extension']);
				if (isset($MIME_TYPE_MAPPINGS[$Extension])) {
					// If we know the extension, let's just use that to get the MIME type: other types of detection are not very reliable.
					if ($PHPOF_DEBUG_MODE) phpof_debug("MIMETypeHelper::getTypeFromFile(): Overriding file-content detection as known extension '$Extension' found");
					return MIMETypeHelper::getTypeFromFilename("x.$Extension");
				}
			}
		}
		
		$Filetype = null;
		
		// Check for PHP mime_content_type() function
		if (function_exists('mime_content_type')) {
			if ($PHPOF_DEBUG_MODE) phpof_debug('MIMETypeHelper::getTypeFromFile(): Using mime_content_type() to try to find MIME type of data');
			// TJ disabled; PHP5 always returns text/plain
			//$Filetype = mime_content_type($File);
			if ($PHPOF_DEBUG_MODE) phpof_debug("MIMETypeHelper::getTypeFromFile(): mime_content_type() returned $Filetype");
		}
		
		// Try to use command-line 'file'
		if (empty($Filetype)) {
			if ($PHPOF_DEBUG_MODE) phpof_debug('MIMETypeHelper::getTypeFromFile(): Attempting to use command-line \'file\' command to try to find MIME type of data');
			
			$ProcDescriptor = array(
				1 => array('pipe', 'w')	// stdout
			);
			$Proc = proc_open("file -ib $File", $ProcDescriptor, $Pipes);
			if (is_resource($Proc)) {
				// Read the output
				$Filetype = trim(fgets($Pipes[1],1024));
				fclose($Pipes[1]);
				proc_close($Proc);
			}
			
			// Some versions of 'file' are buggy and don't always actually return a MIME type (e.g. with PDFs)
			// Check that the output from 'file' has a slash in it; if not, it's junk
			if (!strstr($Filetype,'/')) {
				$Filetype = null;
			}
			
			$Filetype = preg_replace("/;.+/",'',$Filetype);
			
			if ($PHPOF_DEBUG_MODE) phpof_debug("MIMETypeHelper::getTypeFromFile():'file' command returned $Filetype");
		}
		
		// Canonicalise if necessary
		if (isset($MIME_TYPE_TRANSFORMATIONS[$Filetype])) {
			$Filetype = $MIME_TYPE_TRANSFORMATIONS[$Filetype];
		}
		
		// Try to get the MIME type from the filename
		if (empty($Filetype) && !empty($UseFilename)) {
			if ($PHPOF_DEBUG_MODE) phpof_debug("MIMETypeHelper::getTypeFromFile(): Attempting to get MIME type from filename '$UseFilename'");
			$Filetype = MIMETypeHelper::getTypeFromFilename(strtolower($UseFilename));
			if ($PHPOF_DEBUG_MODE) phpof_debug("MIMETypeHelper::getTypeFromFile(): getTypeFromFilename() returned '$Filetype'");
		}
		
		// Final fallback if all else fails
		if (empty($Filetype)) {
			$Filetype = 'application/octet-stream';
		}
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("MIMETypeHelper::getTypeFromFile(): Final decision is $Filetype");
		
		return $Filetype;
	}
	
	/**
	* @desc Get MIME type for a given filename, based on its extension
	* @param $Filename Filename
	* @return string MIME type
	*/
	function getTypeFromFilename($Filename)
	{
		global $MIME_TYPE_MAPPINGS, $PHPOF_DEBUG_MODE;
		
		$PathParts = pathinfo($Filename);
		if (isset($PathParts['extension'])) {
			$Extension = $PathParts['extension'];
		}
		if ($PHPOF_DEBUG_MODE) phpof_debug("MIMETypeHelper::getTypeFromFilename(): Found extension '$Extension'");
		
		if (isset($Extension) && !empty($MIME_TYPE_MAPPINGS[$Extension])) {
			return $MIME_TYPE_MAPPINGS[$Extension];
		} else {
			return 'application/octet-stream';
		}
	}
	
	function getFileExtensionFromMimeType($MIMEType)
	{
		global $MIME_TYPE_MAPPINGS;
		
		foreach ($MIME_TYPE_MAPPINGS as $Extension => $Type)
		{
			if ($Type == $MIMEType) return $Extension;
		}
		if (defined(FILE_EXTENSION_UNKNOWN)) {
			return FILE_EXTENSION_UNKNOWN;
		} else {
			return false;
		}
	}
	
	function getMimeGroupFromMimeType($MIMEType)
	{
		if (preg_match("/(.+)\/(.+)/",$MIMEType, $Matches)) {
			return $Matches[1];
		} else {
			trigger_error("Invalid MIME type: $MIMEType",E_USER_WARNING);
			return null;
		}
	}
	
	function getNameFromMimeType($MIMEType)
	{
		global $MIME_TYPE_NAMES;
		if (isset($MIME_TYPE_NAMES[$MIMEType])) {
			return $MIME_TYPE_NAMES[$MIMEType];
		}
		return 'Unknown file type';
	}
}
