<?php

// bsmtp.class.php - provide BSMTP functions
//
// Version $Id: bsmtp.class.php 1584 2009-04-09 11:13:53Z tim $
//
// (c)2002-2009 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require_once('XValidate.class.php');

class BSMTP {
	var $FileHandle;
	var $TmpFilename;
	var $SaveBatchFile;
	
	function BSMTP ($TmpDir = '/tmp')
	{
		$TmpFilename = md5(uniqid('',1));
		$TmpFile = "$TmpDir/bsmtp.$TmpFilename";
		$this->TmpFilename = $TmpFile;
		if (!$this->FileHandle = fopen($TmpFile,'w')) trigger_error("Could not open BSMTP temp file $this->TmpFile",E_USER_ERROR);
	}
	
	function addMail ($To, $Subject, $Message, $EnvelopeSender, $Headers='')
	{
		if (substr($Headers,-1) != "\n") $Headers .= "\n";
		$SMTPMessage = 
			$Headers.
			"Subject: $Subject\n".
			"Date: ".date('r')."\n".
			"\n".
			$Message;
		$this->addRawMail($To, $SMTPMessage, $EnvelopeSender);
	}
	
	function addRawMail ($To, $Message, $EnvelopeSender)
	{
		$Recipients = explode(',',$To);
		
		$EnvelopeRecipientArray = array();
		foreach ($Recipients as $Recipient) {
			$Recipient = trim($Recipient);
			if (!empty($Recipient)) {
				if (XValidate::Email($Recipient) != XVALIDATE_SYNTAXERR) {
					$EnvelopeRecipientArray[] = $Recipient;
				} else {
					// DEBUG
					//print "WARNING: bad recipient $Recipient";
				}
			}
		}
		
		// Normalise message to use UNIX newlines. Although SMTP itself uses \r\n,
		// Exim is happy with \n as BSMTP input
		$Message = str_replace("\r\n", "\n", $Message);
		
		if (count($EnvelopeRecipientArray) > 0) {
			fputs($this->FileHandle, "MAIL FROM:<$EnvelopeSender>\n");
			foreach ($EnvelopeRecipientArray as $Recipient) {
				fputs($this->FileHandle, "RCPT TO:<$Recipient>\n");
			}
			fputs($this->FileHandle, "DATA\n");
			$MessageLines = explode("\n", $Message);
			foreach ($MessageLines as $Line) {
				if ($Line == '.') $Line = '..'; // Escape single full stops on their own line as per RFC2821, to avoid truncated messages
				fputs($this->FileHandle, "$Line\n");
			}
			fputs($this->FileHandle, ".\n");
		}
	}
	
	/**
	 * Get the filename of the file containing the BSMTP output
	 *
	 * @return string Full filesystem path to BSMTP file
	 */
	public function getBsmtpFile()
	{
		return $this->TmpFilename;
	}

	function sendQueue ()
	{
		fclose($this->FileHandle);
		
		$FP = fopen($this->TmpFilename,'r');
		if ($FP) {
			$ProcDescriptor = array(
				0 => array('pipe', 'r'),	// stdin
				1 => array('pipe', 'w'),	// stdout
				2 => array('pipe', 'w'),	// stderr
			);
			$Command = "/usr/sbin/exim -bS";
			$Proc = proc_open($Command,$ProcDescriptor,$Pipes);
			if (is_resource($Proc)) {
				$CmdOutput = '';
				while (!feof($FP)) {
					$Line = fgets($FP, 1024);
					fputs($Pipes[0], $Line);
				}
				fclose($Pipes[0]);
				while(!feof($Pipes[1])) {
					$CmdOutput .= fgets($Pipes[1], 1024);
				}
				fclose($Pipes[1]);
				while(!feof($Pipes[2])) {
					$CmdOutput .= fgets($Pipes[2], 1024);
				}
				fclose($Pipes[2]);
				
				$RetVal = proc_close($Proc);
				if ($RetVal != 0) {
					trigger_error("BSMTP error: status $RetVal, output: $CmdOutput (command was: $Command)",E_USER_ERROR);
					return false;
				} else if (!empty($CmdOutput)) {
					trigger_error("BSMTP warning: $CmdOutput (command was: $Command)",E_USER_WARNING);
					return false;
				}
			} else {
				trigger_error('BSMTP error: could not open proc',E_USER_ERROR);
				return false;
			}
			if (!$this->SaveBatchFile) {
				unlink($this->TmpFilename);
			}
			fclose($FP);
		} else {
			trigger_error("Could not open temp file $this->TmpFilename for reading",E_USER_ERROR);
			return false;
		}
		return true;
	}
}
